@@ -1,5 +1,6 @@
 #imports
 import datetime
+today = datetime.date.today()
 from pathlib import Path
 
 from vinca import reviewers, editors, schedulers 
@@ -51,39 +52,30 @@ class Card:
 		(self.path/'config').write_text(f'{editor} {reviewer} {scheduler}')
 
 	def set_due_date(self, date):
-		assert type(date)==datetime.date or date in ['deleted','suspended']
+		assert type(date)==datetime.date
 		(self.path/'due_date').write_text(str(date))
 
-	def add_tag(self, tag):
-		tags = self.tags | {tag}
+	def add_tags(self, tags):
+		tags = self.tags | set(tags)
 		(self.path/'tags').write_text(' '.join(tags))
 
-	def remove_tag(self, tag):
-		tags = self.tags - {tag}
+	def remove_tags(self, tags):
+		tags = self.tags - set(tags)
 		(self.path/'tags').write_text(' '.join(self.tags))
 
 	def add_history(self, date, time, grade):
-		assert type(date) == datetime.date
-		assert type(grade) == int and type(time) == int
+		assert type(date) == datetime.date and type(grade) == type(time) == int
 		with (self.path/'history').open('a') as f:
 			f.write(f'{date} {time} {grade}\n')
 
 	def undo_history(self):
 		new_history = self.history[:-1]
-		(self.path/'history').open('r+').truncate() # clear the history
+		(self.path/'history').write_text('') # clear the history
 		for date, time, grade in new_history:
 			self.add_history(date, time, grade) 
 		if not self.history:
-			self.add_history(datetime.date.today(),30,0)
+			self.add_history(today,30,0)
 			
-
-	def auxfile(self, filename, mode='r'):
-		# aux file stands for auxilliary file
-		# this is the interface for all file manipulation
-		# TODO: no it's not! normally I just reach into the card folder to access files.
-		assert filename in self.auxfiles
-		return (self.path/filename).open(mode)
-
 	def make_auxfile(self, filename):
 		(self.path/filename).touch()
 
@@ -101,10 +93,10 @@ class Card:
 		return schedulers.schedule(self) 
 
 	def delete(self):
-		self.set_due_date('deleted')
+		self.set_due_date(datetime.date(year=3000,month=1,day=1))
 
 	def undelete(self):
-		if self.last_grade == -1:
+		while self.last_grade == -1:
 			self.undo_history()
 		schedulers.schedule(self)
 
@@ -133,10 +125,8 @@ class Card:
 	@property
 	def due_date(self):
 		dd = (self.path/'due_date').read_text()
-		if dd in ['deleted', 'suspended']:
-			return dd
-		year, month, day = [int(x) for x in dd.split('-')]
-		return datetime.date(year, month, day)
+		assert dd, f'The due_date file for card {self.id} is empty.'
+		return datetime.date(*[int(x) for x in dd.split('-')])
 
 	@property
 	def create_date(self):
@@ -156,7 +146,7 @@ class Card:
 		return len(self.history) == 1
 	@property
 	def deletedQ(self):
-		return self.due_date == 'deleted'
+		return self.due_date >= datetime.date(year=3000,month=1,day=1)
 	@property
 	def suspendedQ(self):
 		return self.due_date == 'suspended'
@@ -201,7 +191,7 @@ class Deckdict(dict):
 	def __init__(self):
 		dict.__init__(self)
 		self.list = []
-		lines = decks_file.read_text().splitlines()
+		lines = [l for l in decks_file.read_text().splitlines() if l]
 		for i,line in enumerate(lines):
 			name, *tags = line.split()
 			include = [t[1:] for t in tags if t[0]=='+']
@@ -214,6 +204,13 @@ class Deckdict(dict):
 			self.list.append(deck)
 
 	def delete(self, deck):
+		assert deck in self.list, f'{[deck]} not in {self.list}'
 		del self[deck.name]
 		del self[deck.id]
-		decks_file.write_text('\n'.join([d for d in list(self) if d!=deck]))
+		self.list.remove(deck)
+		print(*self.list, sep='\n', file=decks_file.open('w'))
+
+
+deckdict = Deckdict()
+collection = Collection()
+collection.cache_tags()
